package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.action.ProjectDeclareActionChoice;
import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.factory.ProjectDeclareGuardFactory.PendingPreQualificationChoiceGuard;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * 项目申报状态机
 * 市本级
 *
 * @author CMM
 * @since 2023/02/07 15:56
 */
@Slf4j
@Component
@EnableStateMachine(name = ProjectStateMachineBuilderImpl.MACHINE_ID)
@RequiredArgsConstructor
public class ProjectStateMachineBuilderImpl implements BaseStateMachineBuilder<Project, ProjectStateChangeEvent, ProjectStatus> {

    public static final String MACHINE_ID = "projectDeclareStateMachine";

    private final BeanFactory beanFactory;

    @Override
    public StateMachine<ProjectStatus, ProjectStateChangeEvent> build() throws StateMachineException {
        StateMachine<ProjectStatus, ProjectStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：" + stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    /**
     * 构建状态机
     *
     * @param beanFactory \
     */
    @Override
    public StateMachine<ProjectStatus, ProjectStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<ProjectStatus, ProjectStateChangeEvent> buildStateMachine(BeanFactory beanFactory) throws Exception {
        StateMachineBuilder.Builder<ProjectStatus, ProjectStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(beanFactory);

        builder.configureStates()
                .withStates()
                .initial(ProjectStatus.UNDER_INTERNAL_AUDIT)
                .choice(ProjectStatus.PENDING_PREQUALIFICATION_CHOICE)
                .choice(ProjectStatus.PREQUALIFICATION_WITHDRAW_CHOICE)
                .states(EnumSet.allOf(ProjectStatus.class));

        builder.configureTransitions()
                // 单位内部审核通过,从单位内部审核中到待预审
                .withExternal()
                .source(ProjectStatus.UNDER_INTERNAL_AUDIT)
                .target(ProjectStatus.PENDING_PREQUALIFICATION)
                .event(ProjectStateChangeEvent.UNDER_INTERNAL_PASS).and()
                // 单位内部审核驳回，从单位内部审核中到单位内部审核不通过
                .withExternal()
                .source(ProjectStatus.UNDER_INTERNAL_AUDIT)
                .target(ProjectStatus.UNDER_INTERNAL_AUDIT_NOT_PASS)
                .event(ProjectStateChangeEvent.UNDER_INTERNAL_REJECT).and()
                // 单位内部审核中撤回，从单位内部审核中到待申报
                .withExternal()
                .source(ProjectStatus.UNDER_INTERNAL_AUDIT)
                .target(ProjectStatus.TO_BE_DECLARED)
                .event(ProjectStateChangeEvent.UNDER_INTERNAL_WITHDRAW).and()
                // 单位内部审核不通过重新提交，从单位内部审核不通过到单位内部审核中
                .withExternal()
                .source(ProjectStatus.UNDER_INTERNAL_AUDIT_NOT_PASS)
                .target(ProjectStatus.UNDER_INTERNAL_AUDIT)
                .event(ProjectStateChangeEvent.UNDER_INTERNAL_REJECT_RESUBMIT).and()
                // 待预审预审申报，从待预审到待预审选择
                .withExternal()
                .source(ProjectStatus.PENDING_PREQUALIFICATION)
                .target(ProjectStatus.PENDING_PREQUALIFICATION_CHOICE)
                .event(ProjectStateChangeEvent.PRELIMINARY_REVIEW_DECLARE).and()
                // 待预审申报，从待预审选择->省级部门联审中，预审中，完成其中一种状态
                .withChoice()
                .source(ProjectStatus.PENDING_PREQUALIFICATION_CHOICE)
                .first(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS, new PendingPreQualificationChoiceGuard(), new ProjectDeclareActionChoice())
                .last(ProjectStatus.PRE_APPLYING, new ProjectDeclareActionChoice()).and()

                // 省级部门联审通过，从省级部门联审中到省级部门联审成功
                .withExternal()
                .source(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS)
                .target(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_SUCCESS)
                .event(ProjectStateChangeEvent.PROVINCIAL_DEPARTMENT_REVIEW_PASS).and()
                // 省级部门联审通过，从省级部门联审成功到预审中
                .withExternal()
                .source(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_SUCCESS)
                .target(ProjectStatus.PRE_APPLYING)
                .event(ProjectStateChangeEvent.PROVINCIAL_PASS_TO_PRELIMINARY_REVIEW).and()
                // 省级部门联审不通过，从省级部门联审中到省级部门联审不通过
                .withExternal()
                .source(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS)
                .target(ProjectStatus.THE_JOINT_REVIEW_OF_PROVINCIAL_DEPARTMENTS_FAILED)
                .event(ProjectStateChangeEvent.PROVINCIAL_DEPARTMENT_REVIEW_REJECT).and()
                // 省级部门联审中撤回，从省级部门联审中到待预审
                .withExternal()
                .source(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS)
                .target(ProjectStatus.PENDING_PREQUALIFICATION)
                .event(ProjectStateChangeEvent.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_WITHDRAW).and()
                // 预审通过，从预审中到部门联审中
                .withExternal()
                .source(ProjectStatus.PRE_APPLYING)
                .target(ProjectStatus.DEPARTMENT_JOINT_REVIEW)
                .event(ProjectStateChangeEvent.PRELIMINARY_REVIEW_PASS).and()
                // 预审驳回，从预审中到预审不通过
                .withExternal()
                .source(ProjectStatus.PRE_APPLYING)
                .target(ProjectStatus.PREQUALIFICATION_FAILED)
                .event(ProjectStateChangeEvent.PRELIMINARY_REVIEW_REJECT).and()
                // 预审中撤回，从预审中到预审中撤回选择态
                .withExternal()
                .source(ProjectStatus.PRE_APPLYING)
                .target(ProjectStatus.PREQUALIFICATION_WITHDRAW_CHOICE)
                .event(ProjectStateChangeEvent.PRE_APPLYING_WITHDRAW).and()
                // 预审中撤回，从预审中撤回选择态->省级部门联审成功，待预审，完成其中一种状态
                .withChoice()
                .source(ProjectStatus.PREQUALIFICATION_WITHDRAW_CHOICE)
                .first(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_SUCCESS, new PendingPreQualificationChoiceGuard()
                        , new ProjectDeclareActionChoice())
                .last(ProjectStatus.PENDING_PREQUALIFICATION, new ProjectDeclareActionChoice()).and()
                // 预审不通过重新提交，从预审不通过到待预审选择
                .withExternal()
                .source(ProjectStatus.PREQUALIFICATION_FAILED)
                .target(ProjectStatus.PENDING_PREQUALIFICATION_CHOICE)
                .event(ProjectStateChangeEvent.PRELIMINARY_REVIEW_REJECT_RESUBMIT).and()
                // 预审不通过重新提交，从待预审选择->省级部门联审中，预审中，完成其中一种状态
                .withChoice()
                .source(ProjectStatus.PENDING_PREQUALIFICATION_CHOICE)
                .first(ProjectStatus.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS, new PendingPreQualificationChoiceGuard(), new ProjectDeclareActionChoice())
                .last(ProjectStatus.PRE_APPLYING, new ProjectDeclareActionChoice()).and()
                // 部门联审通过，从部门联审中到年度计划中
                .withExternal()
                .source(ProjectStatus.DEPARTMENT_JOINT_REVIEW)
                .target(ProjectStatus.IN_THE_ANNUAL_PLAN)
                .event(ProjectStateChangeEvent.DEPARTMENT_UNITED_REVIEW_PASS).and()
                // 部门联审驳回，从部门联审中到部门联审不通过
                .withExternal()
                .source(ProjectStatus.DEPARTMENT_JOINT_REVIEW)
                .target(ProjectStatus.DEPARTMENT_JOINT_REVIEW_FAILED)
                .event(ProjectStateChangeEvent.DEPARTMENT_UNITED_REVIEW_REJECT).and()
                // 部门联审中撤回，从部门联审中到预审中
                .withExternal()
                .source(ProjectStatus.DEPARTMENT_JOINT_REVIEW)
                .target(ProjectStatus.PRE_APPLYING)
                .event(ProjectStateChangeEvent.DEPARTMENT_JOINT_REVIEW_WITHDRAW).and()
                // 年度计划中开启方案申报，从年度计划中到方案待申报
                .withExternal()
                .source(ProjectStatus.IN_THE_ANNUAL_PLAN)
                .target(ProjectStatus.PLAN_TO_BE_DECLARED)
                .event(ProjectStateChangeEvent.ANNUAL_PLAN_PROJECT_OPEN_PLAN_DECLARE).and()
                // 年度计划中暂缓，从年度计划中到被暂缓
                .withExternal()
                .source(ProjectStatus.IN_THE_ANNUAL_PLAN)
                .target(ProjectStatus.BE_SUSPENDED)
                .event(ProjectStateChangeEvent.ANNUAL_PLAN_SUSPEND).and()
                // 方案待申报申报方案，从方案待申报到方案评审中
                .withExternal()
                .source(ProjectStatus.PLAN_TO_BE_DECLARED)
                .target(ProjectStatus.SCHEME_UNDER_REVIEW)
                .event(ProjectStateChangeEvent.DECLARE_PLAN).and()
                // 方案评审通过，从方案评审中到待立项批复
                .withExternal()
                .source(ProjectStatus.SCHEME_UNDER_REVIEW)
                .target(ProjectStatus.TO_BE_APPROVED)
                .event(ProjectStateChangeEvent.PLAN_REVIEW_PASS).and()
                // 方案评审驳回，从方案评审中到方案评审不通过
                .withExternal()
                .source(ProjectStatus.SCHEME_UNDER_REVIEW)
                .target(ProjectStatus.SCHEME_REVIEW_FAILED)
                .event(ProjectStateChangeEvent.PLAN_REVIEW_REJECT).and()
                // 方案评审中撤回，从方案评审中到方案待申报
                .withExternal()
                .source(ProjectStatus.SCHEME_UNDER_REVIEW)
                .target(ProjectStatus.PLAN_TO_BE_DECLARED)
                .event(ProjectStateChangeEvent.SCHEME_UNDER_REVIEW_WITHDRAW).and()
                // 方案评审不通过重新提交，从方案评审不通过到方案评审中
                .withExternal()
                .source(ProjectStatus.SCHEME_REVIEW_FAILED)
                .target(ProjectStatus.SCHEME_UNDER_REVIEW)
                .event(ProjectStateChangeEvent.PLAN_REVIEW_REJECT_RESUBMIT).and()
                // 立项备案
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD)
                .target(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_SUBMIT).and()
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_RECORD_FAILED)
                .target(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_RESUBMIT).and()
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .target(ProjectStatus.TO_BE_PURCHASED)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_PASS).and()
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .target(ProjectStatus.DECLARED_APPROVED_RECORD_FAILED)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_FAILED).and()
                // 待立项批复批复，从待立项批复到待采购
                .withExternal()
                .source(ProjectStatus.TO_BE_APPROVED)
                .target(ProjectStatus.TO_BE_PURCHASED)
                .event(ProjectStateChangeEvent.PROJECT_APPROVAL).and()
                // 待采购采购备案，从待采购到实施中
                .withExternal()
                .source(ProjectStatus.TO_BE_PURCHASED)
                .target(ProjectStatus.ON_PURCHASING)
                .event(ProjectStateChangeEvent.SUBMIT_PURCHASE_NOTICE).and()
                // 填写试试计划 -> 待初验
                .withExternal()
                .source(ProjectStatus.ON_PURCHASING)
                .target(ProjectStatus.TO_BE_FIRST_INSPECTED)
                .event(ProjectStateChangeEvent.SUBMIT_OPERATION_PLAN).and()
                // 上传初验材料
                .withExternal()
                .source(ProjectStatus.TO_BE_FIRST_INSPECTED)
                .target(ProjectStatus.ON_PILOT_RUNNING)
                .event(ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES).and()
                // 待终验终验申请，从待终验到终验审核中
                .withExternal()
                .source(ProjectStatus.ON_PILOT_RUNNING)
                .target(ProjectStatus.ON_FINALLY_INSPECTED)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_APPLICATION).and()
                // 终审审核通过，从终审审核中到已归档
                .withExternal()
                .source(ProjectStatus.ON_FINALLY_INSPECTED)
                .target(ProjectStatus.ARCHIVED)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS).and()
                // 终审审核不通过，从终审审核中到终审审核不通过
                .withExternal()
                .source(ProjectStatus.ON_FINALLY_INSPECTED)
                .target(ProjectStatus.FINALLY_INSPECTED_FAILED)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_REJECT).and()
                // 终审审核中撤回，从终审审核中到待终验
                .withExternal()
                .source(ProjectStatus.ON_FINALLY_INSPECTED)
                .target(ProjectStatus.ON_PILOT_RUNNING)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_IS_UNDER_REVIEW_WITHDRAW).and()
                // 终验中 重新提交
                .withExternal()
                .source(ProjectStatus.FINALLY_INSPECTED_FAILED)
                .target(ProjectStatus.ON_FINALLY_INSPECTED)
                .event(ProjectStateChangeEvent.FINAL_RE_ACCEPTANCE_APPLICATION).and();
        return builder.build();
    }

    /**
     * 持久化配置
     *
     * @author CMM
     * @since 2023/02/07 16:22
     */
    @Bean(name = "projectDeclareStatePersister")
    @Override
    public StateMachinePersister<ProjectStatus, ProjectStateChangeEvent, Project> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<ProjectStatus, ProjectStateChangeEvent, Project>() {
            @Override
            public void write(StateMachineContext<ProjectStatus, ProjectStateChangeEvent> context, Project contextObj) {
                log.info("当前项目为：{}", contextObj);
            }

            @Override
            public StateMachineContext<ProjectStatus, ProjectStateChangeEvent> read(Project contextObj) {
                ProjectStatus status = ProjectStatus.getNoNull(contextObj.getStatus());
                return new DefaultStateMachineContext<>(status, null, null, null, null, MACHINE_ID);
            }
        });
    }

}
